import * as Fragment from '@corvu/utils/components/Fragment';
import Fragment__default from '@corvu/utils/components/Fragment';
import * as solid_js from 'solid-js';
import { ValidComponent, Accessor, Setter, JSX, Component } from 'solid-js';
import { ContentCorvuProps, ContentSharedElementProps, ContentElementProps, RootProps, RootChildrenProps, DynamicProps as DynamicProps$1, TriggerCorvuProps, TriggerSharedElementProps, TriggerElementProps, ContextValue } from '@corvu/disclosure';
export { ContextValue as DisclosureContextValue, useContext as useDisclosureContext } from '@corvu/disclosure';
import { DynamicProps } from '@corvu/utils/dynamic';
export { DynamicProps } from '@corvu/utils/dynamic';
import { ElementOf } from '@corvu/utils/dom';

type AccordionContentCorvuProps = ContentCorvuProps;
type AccordionContentSharedElementProps<T extends ValidComponent = 'div'> = ContentSharedElementProps<T>;
type AccordionContentElementProps = AccordionContentSharedElementProps & {
    role: 'region';
    'aria-labelledby': string | undefined;
    'data-corvu-accordion-content': '';
} & ContentElementProps;
type AccordionContentProps<T extends ValidComponent = 'div'> = AccordionContentCorvuProps & Partial<AccordionContentSharedElementProps<T>>;
/** Content of an accordion item. Can be animated.
 *
 * @data `data-corvu-accordion-content` - Present on every accordion item content element.
 * @data `data-expanded` - Present when the accordion item is expanded.
 * @data `data-collapsed` - Present when the accordion item is collapsed.
 * @css `--corvu-disclosure-content-width` - The width of the accordion item content. Useful if you want to animate its width.
 * @css `--corvu-disclosure-content-height` - The height of the accordion item content. Useful if you want to animate its height.
 */
declare const AccordionContent: <T extends ValidComponent = "div">(props: DynamicProps<T, AccordionContentProps<T>>) => solid_js.JSX.Element;

type AccordionContextValue = {
    /** Whether multiple accordion items can be expanded at the same time. */
    multiple: Accessor<boolean>;
    /** The value of the accordion. */
    value: Accessor<string | string[] | null>;
    /** Callback fired when the value changes. */
    setValue: Setter<string | string[] | null>;
    /** Whether the accordion can be fully collapsed. */
    collapsible: Accessor<boolean>;
    /** Whether the accordion is disabled. */
    disabled: Accessor<boolean>;
    /** The orientation of the accordion. */
    orientation: Accessor<'horizontal' | 'vertical'>;
    /** Whether the accordion should loop when navigating with the keyboard. */
    loop: Accessor<boolean>;
    /** The text direction of the accordion. */
    textDirection: Accessor<'ltr' | 'rtl'>;
    /** Whether the accordion item content should be removed or hidden when collapsed. */
    collapseBehavior: Accessor<'remove' | 'hide'>;
};
/** Context which exposes various properties to interact with the accordion. Optionally provide a contextId to access a keyed context. */
declare const useAccordionContext: (contextId?: string) => AccordionContextValue;

type AccordionItemCorvuProps = {
    /**
     * Value of the accordion item.
     * @defaultValue `createUniqueId()`
     */
    value?: string;
    /**
     * Whether the accordion item is disabled. Used to override the default provided by `<Accordion.Root>`.
     */
    disabled?: boolean;
    /**
     * The `id` attribute of the accordion item trigger element.
     * @defaultValue `createUniqueId()`
     */
    triggerId?: string;
} & Omit<RootProps, 'expanded' | 'onExpandedChange' | 'initialExpanded' | 'children'>;
type AccordionItemSharedElementProps<T extends ValidComponent = typeof Fragment__default> = {
    children: JSX.Element | ((props: AccordionItemChildrenProps & RootChildrenProps) => JSX.Element);
};
type AccordionItemElementProps = AccordionItemSharedElementProps & {
    'data-corvu-accordion-item': '';
};
type AccordionItemProps<T extends ValidComponent = typeof Fragment__default> = AccordionItemCorvuProps & Partial<AccordionItemSharedElementProps<T>>;
/** Props that are passed to the Item component children callback. */
type AccordionItemChildrenProps = {
    /** Value of the accordion item. */
    value: string;
    /** Whether the accordion item is disabled. */
    disabled: boolean;
    /** The `id` attribute of the accordion item trigger element. */
    triggerId: string | undefined;
};
/** Context wrapper for the accordion item. Is required for every accordion item you create.
 *
 * @data `data-corvu-accordion-item` - Present if the item isn't rendered as a Fragment.
 */
declare const AccordionItem: <T extends ValidComponent = Fragment__default>(props: DynamicProps$1<T, AccordionItemProps<T>>) => JSX.Element;

type AccordionItemContextValue = {
    /** Value of the accordion item. */
    value: Accessor<string>;
    /** Whether the accordion item is disabled. */
    disabled: Accessor<boolean>;
    /** The `id` attribute of the accordion item trigger element. */
    triggerId: Accessor<string | undefined>;
};
/** Context which exposes various properties to interact with the accordion. Optionally provide a contextId to access a keyed context. */
declare const useAccordionItemContext: (contextId?: string) => AccordionItemContextValue;

type AccordionRootProps = {
    /**
     * Whether multiple accordion items can be expanded at the same time.
     * @defaultValue `false`
     */
    multiple?: boolean;
    /**
     * The value of the accordion.
     */
    value?: string | string[] | null;
    /**
     * Callback fired when the value changes.
     */
    onValueChange?: (value: string | string[] | null) => void;
    /**
     * The value of the accordion initially.
     * @defaultValue `null`
     */
    initialValue?: string | string[] | null;
    /**
     * Whether the accordion can be fully collapsed.
     * @defaultValue `true`
     */
    collapsible?: boolean;
    /**
     * Whether the accordion is disabled.
     * @defaultValue `false`
     */
    disabled?: boolean;
    /**
     * The orientation of the accordion.
     * @defaultValue `vertical`
     */
    orientation?: 'vertical' | 'horizontal';
    /**
     * Whether the accordion should loop when navigating with the keyboard.
     * @defaultValue `true`
     */
    loop?: boolean;
    /**
     * The text direction of the accordion.
     * @defaultValue `ltr`
     */
    textDirection?: 'ltr' | 'rtl';
    /**
     * Whether the accordion content should be removed or hidden when collapsed. Useful if you want to always render the content for SEO reasons.
     * @defaultValue `remove`
     */
    collapseBehavior?: 'remove' | 'hide';
    /**
     * The `id` of the accordion context. Useful if you have nested accordions and want to create components that belong to a accordion higher up in the tree.
     */
    contextId?: string;
    /** @hidden */
    children: JSX.Element | ((props: AccordionRootChildrenProps) => JSX.Element);
};
/** Props that are passed to the Root component children callback. */
type AccordionRootChildrenProps = {
    /** Whether multiple accordion items can be expanded at the same time. */
    multiple: boolean;
    /** The value of the accordion. */
    value: string | string[] | null;
    /** Callback fired when the value changes. */
    setValue: Setter<string | string[] | null>;
    /** Whether the accordion can be fully collapsed. */
    collapsible: boolean;
    /** Whether the accordion is disabled. */
    disabled: boolean;
    /** The orientation of the accordion. */
    orientation: 'horizontal' | 'vertical';
    /** Whether the accordion should loop when navigating with the keyboard. */
    loop: boolean;
    /** The text direction of the accordion. */
    textDirection: 'ltr' | 'rtl';
    /** Whether the accordion content should be removed or hidden when collapsed. */
    collapseBehavior: 'remove' | 'hide';
};
/** Context wrapper for the accordion. Is required for every accordion you create. */
declare const AccordionRoot: Component<AccordionRootProps>;

type AccordionTriggerCorvuProps = TriggerCorvuProps;
type AccordionTriggerSharedElementProps<T extends ValidComponent = 'button'> = {
    onKeyDown: JSX.EventHandlerUnion<ElementOf<T>, KeyboardEvent>;
    onFocus?: JSX.EventHandlerUnion<ElementOf<T>, FocusEvent>;
    disabled: boolean | undefined;
} & TriggerSharedElementProps<T>;
type AccordionTriggerElementProps = AccordionTriggerSharedElementProps & {
    id: string | undefined;
    'aria-disabled': 'true' | undefined;
    'data-disabled': '' | undefined;
    'data-corvu-accordion-trigger': '';
} & TriggerElementProps;
type AccordionTriggerProps<T extends ValidComponent = 'button'> = AccordionTriggerCorvuProps & Partial<AccordionTriggerSharedElementProps<T>>;
/** Button that changes the open state of the accordion item when clicked.
 *
 * @data `data-corvu-accordion-trigger` - Present on every accordion trigger element.
 * @data `data-expanded` - Present when the accordion is expanded.
 * @data `data-collapsed` - Present when the accordion is collapsed.
 * @data `data-disabled` - Present when the accordion trigger is disabled.
 */
declare const AccordionTrigger: <T extends ValidComponent = "button">(props: DynamicProps<T, AccordionTriggerProps<T>>) => JSX.Element;

declare const Accordion: solid_js.Component<AccordionRootProps> & {
    Item: <T extends solid_js.ValidComponent = Fragment.default>(props: DynamicProps<T, AccordionItemProps<T>>) => solid_js.JSX.Element;
    Trigger: <T extends solid_js.ValidComponent = "button">(props: DynamicProps<T, AccordionTriggerProps<T>>) => solid_js.JSX.Element;
    Content: <T extends solid_js.ValidComponent = "div">(props: DynamicProps<T, AccordionContentProps<T>>) => solid_js.JSX.Element;
    useContext: (contextId?: string) => AccordionContextValue;
    useItemContext: (contextId?: string) => AccordionItemContextValue;
    useDisclosureContext: (contextId?: string) => ContextValue;
};

export { AccordionContent as Content, type AccordionContentCorvuProps as ContentCorvuProps, type AccordionContentElementProps as ContentElementProps, type AccordionContentProps as ContentProps, type AccordionContentSharedElementProps as ContentSharedElementProps, type AccordionContextValue as ContextValue, AccordionItem as Item, type AccordionItemChildrenProps as ItemChildrenProps, type AccordionItemContextValue as ItemContextValue, type AccordionItemCorvuProps as ItemCorvuProps, type AccordionItemElementProps as ItemElementProps, type AccordionItemProps as ItemProps, type AccordionItemSharedElementProps as ItemSharedElementProps, AccordionRoot as Root, type AccordionRootChildrenProps as RootChildrenProps, type AccordionRootProps as RootProps, AccordionTrigger as Trigger, type AccordionTriggerCorvuProps as TriggerCorvuProps, type AccordionTriggerElementProps as TriggerElementProps, type AccordionTriggerProps as TriggerProps, type AccordionTriggerSharedElementProps as TriggerSharedElementProps, Accordion as default, useAccordionContext as useContext, useAccordionItemContext as useItemContext };
