import { createComponent, mergeProps } from 'solid-js/web';
import { createContext, useContext as useContext$1, createMemo, mergeProps as mergeProps$1, createUniqueId, splitProps, createSignal, untrack, createEffect, onCleanup } from 'solid-js';
import Disclosure, { useContext } from '@corvu/disclosure';
export { useContext as useDisclosureContext } from '@corvu/disclosure';
import { useKeyedContext, createKeyedContext } from '@corvu/utils/create/keyedContext';
import createOnce from '@corvu/utils/create/once';
import createRegister from '@corvu/utils/create/register';
import { Dynamic } from '@corvu/utils/dynamic';
import Fragment from '@corvu/utils/components/Fragment';
import { dataIf, isFunction } from '@corvu/utils';
import createControllableSignal from '@corvu/utils/create/controllableSignal';
import { createList } from 'solid-list';
import { sortByDocumentPosition, callEventHandler } from '@corvu/utils/dom';
import { mergeRefs } from '@corvu/utils/reactivity';

// src/Content.tsx
var AccordionItemContext = createContext();
var createAccordionItemContext = (contextId) => {
  if (contextId === void 0) return AccordionItemContext;
  const context = createKeyedContext(
    `accordion-item-${contextId}`
  );
  return context;
};
var useAccordionItemContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext$1(AccordionItemContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion Item context not found. Make sure to wrap Accordion Item components in <Accordion.Item>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(
    `accordion-item-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion Item context with id "${contextId}" not found. Make sure to wrap Accordion Item components in <Accordion.Item contextId="${contextId}">`
    );
  }
  return context;
};
var InternalAccordionItemContext = createContext();
var createInternalAccordionItemContext = (contextId) => {
  if (contextId === void 0) return InternalAccordionItemContext;
  const context = createKeyedContext(
    `accordion-item-internal-${contextId}`
  );
  return context;
};
var useInternalAccordionItemContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext$1(InternalAccordionItemContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion context not found. Make sure to wrap Accordion components in <Accordion.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(
    `accordion-item-internal-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion context with id "${contextId}" not found. Make sure to wrap Accordion components in <Accordion.Root contextId="${contextId}">`
    );
  }
  return context;
};

// src/Content.tsx
var AccordionContent = (props) => {
  const context = createMemo(() => useInternalAccordionItemContext(props.contextId));
  return createComponent(Disclosure.Content, mergeProps({
    role: "region",
    get ["aria-labelledby"]() {
      return context().triggerId();
    },
    "data-corvu-accordion-content": "",
    "data-corvu-disclosure-content": null
  }, props));
};
var Content_default = AccordionContent;
var AccordionContext = createContext();
var createAccordionContext = (contextId) => {
  if (contextId === void 0) return AccordionContext;
  const context = createKeyedContext(
    `accordion-${contextId}`
  );
  return context;
};
var useAccordionContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext$1(AccordionContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion context not found. Make sure to wrap Accordion components in <Accordion.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(
    `accordion-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion context with id "${contextId}" not found. Make sure to wrap Accordion components in <Accordion.Root contextId="${contextId}">`
    );
  }
  return context;
};
var InternalAccordionContext = createContext();
var createInternalAccordionContext = (contextId) => {
  if (contextId === void 0) return InternalAccordionContext;
  const context = createKeyedContext(
    `accordion-internal-${contextId}`
  );
  return context;
};
var useInternalAccordionContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext$1(InternalAccordionContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion context not found. Make sure to wrap Accordion components in <Accordion.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(
    `accordion-internal-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion context with id "${contextId}" not found. Make sure to wrap Accordion components in <Accordion.Root contextId="${contextId}">`
    );
  }
  return context;
};
var AccordionItem = (props) => {
  const defaultedProps = mergeProps$1({
    accordionId: createUniqueId()
  }, props);
  const [localProps, otherProps] = splitProps(defaultedProps, ["value", "disabled", "collapseBehavior", "triggerId", "contextId", "children"]);
  const [triggerId, registerTriggerId, unregisterTriggerId] = createRegister({
    value: () => localProps.triggerId ?? createUniqueId()
  });
  const context = createMemo(() => useInternalAccordionContext(localProps.contextId));
  const value = createMemo(() => localProps.value ?? createUniqueId());
  const expanded = createMemo(() => context().internalValue().includes(value()));
  const disabled = createMemo(() => localProps.disabled ?? context().disabled());
  const collapseBehavior = createMemo(() => localProps.collapseBehavior ?? context().collapseBehavior());
  const childrenProps = {
    get value() {
      return value();
    },
    get disabled() {
      return disabled();
    },
    get triggerId() {
      return triggerId();
    }
  };
  const memoizedChildren = createOnce(() => localProps.children);
  const resolveChildren = (disclosureChildrenProps) => {
    const children = memoizedChildren()();
    if (isFunction(children)) {
      const mergedProps = mergeProps$1(disclosureChildrenProps, childrenProps);
      return children(mergedProps);
    }
    return children;
  };
  const memoizedAccordionItem = createMemo(() => {
    const AccordionItemContext2 = createAccordionItemContext(localProps.contextId);
    const InternalAccordionItemContext2 = createInternalAccordionItemContext(localProps.contextId);
    return createComponent(AccordionItemContext2.Provider, {
      value: {
        value,
        disabled,
        triggerId
      },
      get children() {
        return createComponent(InternalAccordionItemContext2.Provider, {
          value: {
            value,
            disabled,
            triggerId,
            registerTriggerId,
            unregisterTriggerId
          },
          get children() {
            return createComponent(Dynamic, mergeProps({
              as: Fragment,
              "data-corvu-accordion-item": ""
            }, otherProps, {
              get children() {
                return createComponent(Disclosure, mergeProps({
                  get expanded() {
                    return expanded();
                  },
                  onExpandedChange: (newExpanded) => {
                    if (newExpanded === expanded() || disabled()) return;
                    context().toggleValue(value());
                  },
                  get collapseBehavior() {
                    return collapseBehavior();
                  },
                  get contextId() {
                    return localProps.contextId;
                  }
                }, otherProps, {
                  children: (disclosureChildrenProps) => resolveChildren(disclosureChildrenProps)
                }));
              }
            }));
          }
        });
      }
    });
  });
  return memoizedAccordionItem;
};
var Item_default = AccordionItem;
var AccordionRoot = (props) => {
  const defaultedProps = mergeProps$1({
    multiple: false,
    initialValue: null,
    collapsible: true,
    disabled: false,
    orientation: "vertical",
    loop: true,
    textDirection: "ltr",
    collapseBehavior: "remove"
  }, props);
  const [value, setValue] = createControllableSignal({
    value: () => defaultedProps.value,
    initialValue: defaultedProps.initialValue,
    onChange: defaultedProps.onValueChange
  });
  const internalValue = createMemo(() => {
    const _value = value();
    if (_value === null) {
      return [];
    }
    if (!Array.isArray(_value)) {
      return [_value];
    }
    return _value;
  });
  const toggleValue = (value2) => {
    const _internalValue = internalValue();
    if (!defaultedProps.multiple) {
      if (_internalValue.includes(value2)) {
        if (!defaultedProps.collapsible) return;
        setValue(null);
      } else {
        setValue(value2);
      }
      return;
    } else {
      if (_internalValue.includes(value2)) {
        if (_internalValue.length === 1 && !defaultedProps.collapsible) return;
        setValue(_internalValue.filter((v) => v !== value2));
      } else {
        setValue([..._internalValue, value2]);
      }
    }
  };
  const [triggers, setTriggers] = createSignal([]);
  const selectableTriggers = createMemo(() => {
    return triggers().filter((element) => !element.hasAttribute("disabled")).sort(sortByDocumentPosition);
  });
  const {
    setActive,
    onKeyDown: onTriggerKeyDown
  } = createList({
    items: () => [...Array(selectableTriggers().length).keys()],
    orientation: () => defaultedProps.orientation,
    loop: () => defaultedProps.loop,
    textDirection: () => defaultedProps.textDirection,
    onActiveChange: (index) => {
      if (index === null) return;
      const trigger = selectableTriggers()[index];
      if (!trigger) return;
      trigger.focus();
    }
  });
  const childrenProps = {
    get multiple() {
      return defaultedProps.multiple;
    },
    get value() {
      return value();
    },
    setValue,
    get collapsible() {
      return defaultedProps.collapsible;
    },
    get disabled() {
      return defaultedProps.disabled;
    },
    get orientation() {
      return defaultedProps.orientation;
    },
    get loop() {
      return defaultedProps.loop;
    },
    get textDirection() {
      return defaultedProps.textDirection;
    },
    get collapseBehavior() {
      return defaultedProps.collapseBehavior;
    }
  };
  const memoizedChildren = createOnce(() => defaultedProps.children);
  const resolveChildren = () => {
    const children = memoizedChildren()();
    if (isFunction(children)) {
      return children(childrenProps);
    }
    return children;
  };
  const memoizedAccordionRoot = createMemo(() => {
    const AccordionContext2 = createAccordionContext(defaultedProps.contextId);
    const InternalAccordionContext2 = createInternalAccordionContext(defaultedProps.contextId);
    return createComponent(AccordionContext2.Provider, {
      value: {
        multiple: () => defaultedProps.multiple,
        value,
        setValue,
        collapsible: () => defaultedProps.collapsible,
        disabled: () => defaultedProps.disabled,
        orientation: () => defaultedProps.orientation,
        loop: () => defaultedProps.loop,
        textDirection: () => defaultedProps.textDirection,
        collapseBehavior: () => defaultedProps.collapseBehavior
      },
      get children() {
        return createComponent(InternalAccordionContext2.Provider, {
          value: {
            multiple: () => defaultedProps.multiple,
            value,
            setValue,
            collapsible: () => defaultedProps.collapsible,
            disabled: () => defaultedProps.disabled,
            orientation: () => defaultedProps.orientation,
            loop: () => defaultedProps.loop,
            textDirection: () => defaultedProps.textDirection,
            collapseBehavior: () => defaultedProps.collapseBehavior,
            internalValue,
            toggleValue,
            registerTrigger: (trigger) => setTriggers((triggers2) => [...triggers2, trigger]),
            unregisterTrigger: (trigger) => setTriggers((triggers2) => triggers2.filter((t) => t !== trigger)),
            onTriggerKeyDown,
            onTriggerFocus: (e) => setActive(selectableTriggers().indexOf(e.target))
          },
          get children() {
            return untrack(() => resolveChildren());
          }
        });
      }
    });
  });
  return memoizedAccordionRoot;
};
var Root_default = AccordionRoot;
var AccordionTrigger = (props) => {
  const [localProps, otherProps] = splitProps(props, ["contextId", "ref", "onKeyDown", "onFocus", "disabled"]);
  const [triggerRef, setTriggerRef] = createSignal(null);
  const accordionContext = createMemo(() => useInternalAccordionContext(localProps.contextId));
  createEffect(() => {
    const trigger = triggerRef();
    const context2 = accordionContext();
    if (trigger) {
      context2.registerTrigger(trigger);
      onCleanup(() => context2.unregisterTrigger(trigger));
    }
  });
  const context = createMemo(() => useInternalAccordionItemContext(localProps.contextId));
  createEffect(() => {
    const _context = context();
    _context.registerTriggerId();
    onCleanup(() => _context.unregisterTriggerId());
  });
  const onKeyDown = (e) => {
    !callEventHandler(localProps.onKeyDown, e) && accordionContext().onTriggerKeyDown(e);
  };
  const onFocus = (e) => {
    callEventHandler(localProps.onFocus, e);
    accordionContext().onTriggerFocus(e);
  };
  return createComponent(Disclosure.Trigger, mergeProps({
    ref(r$) {
      var _ref$ = mergeRefs(localProps.ref, setTriggerRef);
      typeof _ref$ === "function" && _ref$(r$);
    },
    onKeyDown,
    onFocus,
    get disabled() {
      return localProps.disabled === true || context().disabled() || void 0;
    },
    get id() {
      return context().triggerId();
    },
    get contextId() {
      return localProps.contextId;
    },
    get ["aria-disabled"]() {
      return context().disabled() ? "true" : void 0;
    },
    get ["data-disabled"]() {
      return dataIf(context().disabled());
    },
    "data-corvu-accordion-trigger": "",
    "data-corvu-disclosure-trigger": null
  }, otherProps));
};
var Trigger_default = AccordionTrigger;

// src/index.ts
var Accordion = Object.assign(Root_default, {
  Item: Item_default,
  Trigger: Trigger_default,
  Content: Content_default,
  useContext: useAccordionContext,
  useItemContext: useAccordionItemContext,
  useDisclosureContext: useContext
});
var src_default = Accordion;

export { Content_default as Content, Item_default as Item, Root_default as Root, Trigger_default as Trigger, src_default as default, useAccordionContext as useContext, useAccordionItemContext as useItemContext };
