// src/Content.tsx
import { createMemo } from "solid-js";
import Disclosure from "@corvu/disclosure";

// src/itemContext.ts
import { createContext, useContext } from "solid-js";
import {
  createKeyedContext,
  useKeyedContext
} from "@corvu/utils/create/keyedContext";
var AccordionItemContext = createContext();
var createAccordionItemContext = (contextId) => {
  if (contextId === void 0) return AccordionItemContext;
  const context = createKeyedContext(
    `accordion-item-${contextId}`
  );
  return context;
};
var useAccordionItemContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext(AccordionItemContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion Item context not found. Make sure to wrap Accordion Item components in <Accordion.Item>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(
    `accordion-item-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion Item context with id "${contextId}" not found. Make sure to wrap Accordion Item components in <Accordion.Item contextId="${contextId}">`
    );
  }
  return context;
};
var InternalAccordionItemContext = createContext();
var createInternalAccordionItemContext = (contextId) => {
  if (contextId === void 0) return InternalAccordionItemContext;
  const context = createKeyedContext(
    `accordion-item-internal-${contextId}`
  );
  return context;
};
var useInternalAccordionItemContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext(InternalAccordionItemContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion context not found. Make sure to wrap Accordion components in <Accordion.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(
    `accordion-item-internal-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion context with id "${contextId}" not found. Make sure to wrap Accordion components in <Accordion.Root contextId="${contextId}">`
    );
  }
  return context;
};

// src/Content.tsx
var AccordionContent = (props) => {
  const context = createMemo(
    () => useInternalAccordionItemContext(props.contextId)
  );
  return <Disclosure.Content
    role="region"
    aria-labelledby={context().triggerId()}
    data-corvu-accordion-content=""
    data-corvu-disclosure-content={null}
    {...props}
  />;
};
var Content_default = AccordionContent;

// src/context.ts
import { createContext as createContext2, useContext as useContext2 } from "solid-js";
import {
  createKeyedContext as createKeyedContext2,
  useKeyedContext as useKeyedContext2
} from "@corvu/utils/create/keyedContext";
var AccordionContext = createContext2();
var createAccordionContext = (contextId) => {
  if (contextId === void 0) return AccordionContext;
  const context = createKeyedContext2(
    `accordion-${contextId}`
  );
  return context;
};
var useAccordionContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext2(AccordionContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion context not found. Make sure to wrap Accordion components in <Accordion.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext2(
    `accordion-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion context with id "${contextId}" not found. Make sure to wrap Accordion components in <Accordion.Root contextId="${contextId}">`
    );
  }
  return context;
};
var InternalAccordionContext = createContext2();
var createInternalAccordionContext = (contextId) => {
  if (contextId === void 0) return InternalAccordionContext;
  const context = createKeyedContext2(
    `accordion-internal-${contextId}`
  );
  return context;
};
var useInternalAccordionContext = (contextId) => {
  if (contextId === void 0) {
    const context2 = useContext2(InternalAccordionContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Accordion context not found. Make sure to wrap Accordion components in <Accordion.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext2(
    `accordion-internal-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Accordion context with id "${contextId}" not found. Make sure to wrap Accordion components in <Accordion.Root contextId="${contextId}">`
    );
  }
  return context;
};

// src/index.ts
import {
  useContext as useDisclosureContext
} from "@corvu/disclosure";

// src/Item.tsx
import {
  createMemo as createMemo2,
  createUniqueId,
  mergeProps,
  splitProps
} from "solid-js";
import createOnce from "@corvu/utils/create/once";
import createRegister from "@corvu/utils/create/register";
import Disclosure2 from "@corvu/disclosure";
import { Dynamic } from "@corvu/utils/dynamic";
import Fragment from "@corvu/utils/components/Fragment";
import { isFunction } from "@corvu/utils";
var AccordionItem = (props) => {
  const defaultedProps = mergeProps(
    {
      accordionId: createUniqueId()
    },
    props
  );
  const [localProps, otherProps] = splitProps(
    defaultedProps,
    [
      "value",
      "disabled",
      "collapseBehavior",
      "triggerId",
      "contextId",
      "children"
    ]
  );
  const [triggerId, registerTriggerId, unregisterTriggerId] = createRegister({
    value: () => localProps.triggerId ?? createUniqueId()
  });
  const context = createMemo2(
    () => useInternalAccordionContext(localProps.contextId)
  );
  const value = createMemo2(() => localProps.value ?? createUniqueId());
  const expanded = createMemo2(() => context().internalValue().includes(value()));
  const disabled = createMemo2(
    () => localProps.disabled ?? context().disabled()
  );
  const collapseBehavior = createMemo2(
    () => localProps.collapseBehavior ?? context().collapseBehavior()
  );
  const childrenProps = {
    get value() {
      return value();
    },
    get disabled() {
      return disabled();
    },
    get triggerId() {
      return triggerId();
    }
  };
  const memoizedChildren = createOnce(() => localProps.children);
  const resolveChildren = (disclosureChildrenProps) => {
    const children = memoizedChildren()();
    if (isFunction(children)) {
      const mergedProps = mergeProps(disclosureChildrenProps, childrenProps);
      return children(mergedProps);
    }
    return children;
  };
  const memoizedAccordionItem = createMemo2(() => {
    const AccordionItemContext2 = createAccordionItemContext(
      localProps.contextId
    );
    const InternalAccordionItemContext2 = createInternalAccordionItemContext(
      localProps.contextId
    );
    return <AccordionItemContext2.Provider
      value={{
        value,
        disabled,
        triggerId
      }}
    >
        <InternalAccordionItemContext2.Provider
      value={{
        value,
        disabled,
        triggerId,
        registerTriggerId,
        unregisterTriggerId
      }}
    >
          <Dynamic
      as={Fragment}
      data-corvu-accordion-item=""
      {...otherProps}
    >
            <Disclosure2
      expanded={expanded()}
      onExpandedChange={(newExpanded) => {
        if (newExpanded === expanded() || disabled()) return;
        context().toggleValue(value());
      }}
      collapseBehavior={collapseBehavior()}
      contextId={localProps.contextId}
      {...otherProps}
    >
              {(disclosureChildrenProps) => resolveChildren(disclosureChildrenProps)}
            </Disclosure2>
          </Dynamic>
        </InternalAccordionItemContext2.Provider>
      </AccordionItemContext2.Provider>;
  });
  return memoizedAccordionItem;
};
var Item_default = AccordionItem;

// src/Root.tsx
import {
  createMemo as createMemo3,
  createSignal,
  mergeProps as mergeProps2,
  untrack
} from "solid-js";
import createControllableSignal from "@corvu/utils/create/controllableSignal";
import { createList } from "solid-list";
import createOnce2 from "@corvu/utils/create/once";
import { isFunction as isFunction2 } from "@corvu/utils";
import { sortByDocumentPosition } from "@corvu/utils/dom";
var AccordionRoot = (props) => {
  const defaultedProps = mergeProps2(
    {
      multiple: false,
      initialValue: null,
      collapsible: true,
      disabled: false,
      orientation: "vertical",
      loop: true,
      textDirection: "ltr",
      collapseBehavior: "remove"
    },
    props
  );
  const [value, setValue] = createControllableSignal({
    value: () => defaultedProps.value,
    initialValue: defaultedProps.initialValue,
    onChange: defaultedProps.onValueChange
  });
  const internalValue = createMemo3(() => {
    const _value = value();
    if (_value === null) {
      return [];
    }
    if (!Array.isArray(_value)) {
      return [_value];
    }
    return _value;
  });
  const toggleValue = (value2) => {
    const _internalValue = internalValue();
    if (!defaultedProps.multiple) {
      if (_internalValue.includes(value2)) {
        if (!defaultedProps.collapsible) return;
        setValue(null);
      } else {
        setValue(value2);
      }
      return;
    } else {
      if (_internalValue.includes(value2)) {
        if (_internalValue.length === 1 && !defaultedProps.collapsible) return;
        setValue(_internalValue.filter((v) => v !== value2));
      } else {
        setValue([..._internalValue, value2]);
      }
    }
  };
  const [triggers, setTriggers] = createSignal([]);
  const selectableTriggers = createMemo3(() => {
    return triggers().filter((element) => !element.hasAttribute("disabled")).sort(sortByDocumentPosition);
  });
  const { setActive, onKeyDown: onTriggerKeyDown } = createList({
    items: () => [...Array(selectableTriggers().length).keys()],
    orientation: () => defaultedProps.orientation,
    loop: () => defaultedProps.loop,
    textDirection: () => defaultedProps.textDirection,
    onActiveChange: (index) => {
      if (index === null) return;
      const trigger = selectableTriggers()[index];
      if (!trigger) return;
      trigger.focus();
    }
  });
  const childrenProps = {
    get multiple() {
      return defaultedProps.multiple;
    },
    get value() {
      return value();
    },
    setValue,
    get collapsible() {
      return defaultedProps.collapsible;
    },
    get disabled() {
      return defaultedProps.disabled;
    },
    get orientation() {
      return defaultedProps.orientation;
    },
    get loop() {
      return defaultedProps.loop;
    },
    get textDirection() {
      return defaultedProps.textDirection;
    },
    get collapseBehavior() {
      return defaultedProps.collapseBehavior;
    }
  };
  const memoizedChildren = createOnce2(() => defaultedProps.children);
  const resolveChildren = () => {
    const children = memoizedChildren()();
    if (isFunction2(children)) {
      return children(childrenProps);
    }
    return children;
  };
  const memoizedAccordionRoot = createMemo3(() => {
    const AccordionContext2 = createAccordionContext(defaultedProps.contextId);
    const InternalAccordionContext2 = createInternalAccordionContext(
      defaultedProps.contextId
    );
    return <AccordionContext2.Provider
      value={{
        multiple: () => defaultedProps.multiple,
        value,
        setValue,
        collapsible: () => defaultedProps.collapsible,
        disabled: () => defaultedProps.disabled,
        orientation: () => defaultedProps.orientation,
        loop: () => defaultedProps.loop,
        textDirection: () => defaultedProps.textDirection,
        collapseBehavior: () => defaultedProps.collapseBehavior
      }}
    >
        <InternalAccordionContext2.Provider
      value={{
        multiple: () => defaultedProps.multiple,
        value,
        setValue,
        collapsible: () => defaultedProps.collapsible,
        disabled: () => defaultedProps.disabled,
        orientation: () => defaultedProps.orientation,
        loop: () => defaultedProps.loop,
        textDirection: () => defaultedProps.textDirection,
        collapseBehavior: () => defaultedProps.collapseBehavior,
        internalValue,
        toggleValue,
        registerTrigger: (trigger) => setTriggers((triggers2) => [...triggers2, trigger]),
        unregisterTrigger: (trigger) => setTriggers((triggers2) => triggers2.filter((t) => t !== trigger)),
        onTriggerKeyDown,
        onTriggerFocus: (e) => setActive(selectableTriggers().indexOf(e.target))
      }}
    >
          {untrack(() => resolveChildren())}
        </InternalAccordionContext2.Provider>
      </AccordionContext2.Provider>;
  });
  return memoizedAccordionRoot;
};
var Root_default = AccordionRoot;

// src/Trigger.tsx
import { callEventHandler } from "@corvu/utils/dom";
import {
  createEffect,
  createMemo as createMemo4,
  createSignal as createSignal2,
  onCleanup,
  splitProps as splitProps2
} from "solid-js";
import Disclosure3 from "@corvu/disclosure";
import { dataIf } from "@corvu/utils";
import { mergeRefs } from "@corvu/utils/reactivity";
var AccordionTrigger = (props) => {
  const [localProps, otherProps] = splitProps2(props, [
    "contextId",
    "ref",
    "onKeyDown",
    "onFocus",
    "disabled"
  ]);
  const [triggerRef, setTriggerRef] = createSignal2(null);
  const accordionContext = createMemo4(
    () => useInternalAccordionContext(localProps.contextId)
  );
  createEffect(() => {
    const trigger = triggerRef();
    const context2 = accordionContext();
    if (trigger) {
      context2.registerTrigger(trigger);
      onCleanup(() => context2.unregisterTrigger(trigger));
    }
  });
  const context = createMemo4(
    () => useInternalAccordionItemContext(localProps.contextId)
  );
  createEffect(() => {
    const _context = context();
    _context.registerTriggerId();
    onCleanup(() => _context.unregisterTriggerId());
  });
  const onKeyDown = (e) => {
    !callEventHandler(localProps.onKeyDown, e) && accordionContext().onTriggerKeyDown(e);
  };
  const onFocus = (e) => {
    callEventHandler(localProps.onFocus, e);
    accordionContext().onTriggerFocus(e);
  };
  return <Disclosure3.Trigger
    ref={mergeRefs(localProps.ref, setTriggerRef)}
    onKeyDown={onKeyDown}
    onFocus={onFocus}
    disabled={localProps.disabled === true || context().disabled() || void 0}
    id={context().triggerId()}
    contextId={localProps.contextId}
    aria-disabled={context().disabled() ? "true" : void 0}
    data-disabled={dataIf(context().disabled())}
    data-corvu-accordion-trigger=""
    data-corvu-disclosure-trigger={null}
    {...otherProps}
  />;
};
var Trigger_default = AccordionTrigger;

// src/index.ts
var Accordion = Object.assign(Root_default, {
  Item: Item_default,
  Trigger: Trigger_default,
  Content: Content_default,
  useContext: useAccordionContext,
  useItemContext: useAccordionItemContext,
  useDisclosureContext
});
var src_default = Accordion;
export {
  Content_default as Content,
  Item_default as Item,
  Root_default as Root,
  Trigger_default as Trigger,
  src_default as default,
  useAccordionContext as useContext,
  useDisclosureContext,
  useAccordionItemContext as useItemContext
};
