import * as solid_js_web from 'solid-js/web';
import { Portal } from 'solid-js/web';
import * as solid_js from 'solid-js';
import { ValidComponent, JSX, Accessor, Setter, ComponentProps, Component } from 'solid-js';
import { DynamicProps, DynamicButtonSharedElementProps, DynamicButtonElementProps } from '@corvu/utils/dynamic';
export { DynamicProps } from '@corvu/utils/dynamic';
import { Ref, ElementOf } from '@corvu/utils/dom';
import { FloatingArrowSharedElementProps, FloatingArrowElementProps } from '@corvu/utils/components/FloatingArrow';
import { Placement, Strategy } from '@floating-ui/dom';
import { FloatingOptions, FloatingState } from '@corvu/utils/create/floating';
export { FloatingOptions, FloatingState } from '@corvu/utils/create/floating';

type TooltipAnchorCorvuProps = {
    /**
     * The `id` of the tooltip context to use.
     */
    contextId?: string;
};
type TooltipAnchorSharedElementProps<T extends ValidComponent = 'div'> = {
    ref: Ref<ElementOf<T>>;
};
type TooltipAnchorElementProps = TooltipAnchorSharedElementProps & {
    'data-corvu-tooltip-anchor': '';
};
type TooltipAnchorProps<T extends ValidComponent = 'div'> = TooltipAnchorCorvuProps & Partial<TooltipAnchorSharedElementProps<T>>;
/** Anchor element to override the floating reference.
 *
 * @data `data-corvu-tooltip-anchor` - Present on every tooltip anchor element.
 */
declare const TooltipAnchor: <T extends ValidComponent = "div">(props: DynamicProps<T, TooltipAnchorProps<T>>) => solid_js.JSX.Element;

type TooltipArrowCorvuProps = {
    /**
     * Size of the arrow in px.
     * @defaultValue 16
     */
    size?: number;
    /**
     * The `id` of the tooltip context to use.
     */
    contextId?: string;
};
type TooltipArrowSharedElementProps<T extends ValidComponent = 'div'> = {
    ref: Ref<ElementOf<T>>;
} & FloatingArrowSharedElementProps<T>;
type TooltipArrowElementProps = TooltipArrowSharedElementProps & {
    'data-corvu-tooltip-arrow': '';
} & FloatingArrowElementProps;
type TooltipArrowProps<T extends ValidComponent = 'div'> = TooltipArrowCorvuProps & Partial<TooltipArrowSharedElementProps<T>>;
/** Arrow element that automatically points towards the floating reference. Comes with a default arrow svg, but can be overridden by providing your own as the children.
 *
 * @data `data-corvu-tooltip-arrow` - Present on every tooltip arrow element.
 */
declare const TooltipArrow: <T extends ValidComponent = "div">(props: DynamicProps<T, TooltipArrowProps<T>>) => solid_js.JSX.Element;

type TooltipContentCorvuProps = {
    /**
     * Whether the tooltip content should be forced to render. Useful when using third-party animation libraries.
     * @defaultValue `false`
     */
    forceMount?: boolean;
    /**
     * The `id` of the tooltip context to use.
     */
    contextId?: string;
};
type TooltipContentSharedElementProps<T extends ValidComponent = 'div'> = {
    ref: Ref<ElementOf<T>>;
    style: string | JSX.CSSProperties;
};
type TooltipContentElementProps = TooltipContentSharedElementProps & {
    id: string;
    role: 'tooltip';
    'data-closed': '' | undefined;
    'data-open': '' | undefined;
    'data-placement': Placement;
    'data-corvu-tooltip-content': '';
};
type TooltipContentProps<T extends ValidComponent = 'div'> = TooltipContentCorvuProps & Partial<TooltipContentSharedElementProps<T>>;
/** Content of the tooltip. Can be animated.
 *
 * @data `data-corvu-tooltip-content` - Present on every tooltip content element.
 * @data `data-open` - Present when the tooltip is open.
 * @data `data-closed` - Present when the tooltip is closed.
 * @data `data-placement` - Current placement of the tooltip.
 */
declare const TooltipContent: <T extends ValidComponent = "div">(props: DynamicProps<T, TooltipContentProps<T>>) => JSX.Element;

type TooltipContextValue = {
    /** Whether the tooltip is open. */
    open: Accessor<boolean>;
    /** Change the open state of the tooltip. */
    setOpen: Setter<boolean>;
    /** The initial placement of the tooltip. */
    placement: Accessor<Placement>;
    /** The strategy to use when positioning the tooltip. */
    strategy: Accessor<Strategy>;
    /** Floating options of the tooltip. */
    floatingOptions: Accessor<FloatingOptions | null>;
    /** The current floating state of the tooltip. */
    floatingState: Accessor<FloatingState>;
    /** The delay in milliseconds before the tooltip opens after the mouse pointer is moved over the trigger. */
    openDelay: Accessor<number>;
    /** The delay in milliseconds before the tooltip closes after the mouse pointer leaves. */
    closeDelay: Accessor<number>;
    /** The duration in milliseconds that the tooltip should ignore the `openDelay` when the mouse pointer leaves and re-enters. */
    skipDelayDuration: Accessor<number>;
    /** Whether the tooltip content should be hoverable. */
    hoverableContent: Accessor<boolean>;
    /** The group this tooltip is in. */
    group: Accessor<true | string | null>;
    /** Whether the tooltip should open when the trigger is focused. */
    openOnFocus: Accessor<boolean>;
    /** Whether the tooltip should open when the mouse pointer is moved over the trigger. */
    openOnHover: Accessor<boolean>;
    /** Whether the tooltip should close when the user presses the `Escape` key. */
    closeOnEscapeKeyDown: Accessor<boolean>;
    /** Whether the tooltip should close when the user presses on the trigger. */
    closeOnPointerDown: Accessor<boolean>;
    /** Whether the tooltip should close when the user scrolls. */
    closeOnScroll: Accessor<boolean>;
    /** Whether the tooltip content is present in the DOM. */
    contentPresent: Accessor<boolean>;
    /** The tooltip content element. */
    contentRef: Accessor<HTMLElement | null>;
    /** The `id` of the tooltip element. */
    tooltipId: Accessor<string>;
};
/** Context which exposes various properties to interact with the tooltip. Optionally provide a contextId to access a keyed context. */
declare const useTooltipContext: (contextId?: string) => TooltipContextValue;

type TooltipPortalProps = {
    /**
     * Whether the tooltip portal should be forced to render. Useful when using third-party animation libraries.
     * @defaultValue `false`
     */
    forceMount?: boolean;
    /**
     * The `id` of the tooltip context to use.
     */
    contextId?: string;
};
/** Portals its children at the end of the body element to ensure that the tooltip always rendered on top. */
declare const TooltipPortal: (props: TooltipPortalProps & ComponentProps<typeof Portal>) => solid_js.JSX.Element;

type TooltipRootProps = {
    /**
     * Whether the tooltip is open.
     */
    open?: boolean;
    /**
     * Callback fired when the open state changes.
     */
    onOpenChange?: (open: boolean) => void;
    /**
     * Whether the tooltip is open initially.
     * @defaultValue `false`
     */
    initialOpen?: boolean;
    /**
     * The initial placement of the tooltip.
     * @defaultValue `'bottom'`
     */
    placement?: Placement;
    /**
     * The strategy to use when positioning the tooltip.
     * @defaultValue `'absolute'`
     */
    strategy?: Strategy;
    /**
     * Floating options of the tooltip.
     * @defaultValue `{ flip: true, shift: true }`
     */
    floatingOptions?: FloatingOptions | null;
    /**
     * The delay in milliseconds before the tooltip opens after the mouse pointer is moved over the trigger.
     * @defaultValue `500`
     */
    openDelay?: number;
    /**
     * The delay in milliseconds before the tooltip closes after the mouse pointer leaves.
     * @defaultValue `0`
     */
    closeDelay?: number;
    /**
     * The duration in milliseconds that the tooltip should ignore the `openDelay` when the mouse pointer leaves and re-enters.
     * @defaultValue `0`
     */
    skipDelayDuration?: number;
    /**
     * Whether the tooltip content should be hoverable.
     * @defaultValue `true`
     */
    hoverableContent?: boolean;
    /**
     * Optionally add this tooltip to a group. With the same `group` property will close when another tooltip in the same group opens.
     */
    group?: true | string;
    /**
     * Whether the tooltip should open when the trigger is focused.
     * @defaultValue `true`
     */
    openOnFocus?: boolean;
    /**
     * Callback fired when the trigger is focused and `openOnFocus` is `true`. Can be prevented by calling `event.preventDefault`.
     */
    onFocus?: (event: FocusEvent) => void;
    /**
     * Callback fired when the trigger is blurred and `openOnFocus` is `true`. Can be prevented by calling `event.preventDefault`.
     */
    onBlur?: (event: FocusEvent) => void;
    /**
     * Whether the tooltip should open when the mouse pointer is moved over the trigger.
     * @defaultValue `true`
     */
    openOnHover?: boolean;
    /**
     * Callback fired when the mouse pointer is moved over the trigger and `openOnHover` is `true`. Can be prevented by calling `event.preventDefault`.
     */
    onHover?: (event: MouseEvent) => void;
    /**
     * Callback fired when the mouse pointer leaves the tooltip and `openOnHover` is `true`. Can be prevented by calling `event.preventDefault`.
     */
    onLeave?: (event: MouseEvent) => void;
    /**
     * Whether the tooltip should close when the user presses the `Escape` key.
     * @defaultValue `true`
     */
    closeOnEscapeKeyDown?: boolean;
    /**
     * Callback fired when the user presses the `Escape` key. Can be prevented by calling `event.preventDefault`.
     */
    onEscapeKeyDown?: (event: KeyboardEvent) => void;
    /**
     * Whether the tooltip should close when the user presses on the trigger.
     * @defaultValue `true`
     */
    closeOnPointerDown?: boolean;
    /**
     * Callback fired when the user presses on the trigger and `closeOnPointerDown` is `true`. Can be prevented by calling `event.preventDefault`.
     */
    onPointerDown?: (event: MouseEvent) => void;
    /**
     * Whether the tooltip should close when the user scrolls.
     * @defaultValue `true`
     */
    closeOnScroll?: boolean;
    /**
     * Callback fired when the user scrolls and the tooltip is about to close. Can be prevented by calling `event.preventDefault`.
     */
    onScroll?: (event: Event) => void;
    /**
     * Callback fired when the tooltip content present state changes.
     */
    onContentPresentChange?: (present: boolean) => void;
    /**
     * The `id` attribute of the tooltip element.
     * @defaultValue `createUniqueId()`
     */
    tooltipId?: string;
    /**
     * The `id` of the tooltip context. Useful if you have nested tooltips and want to create components that belong to a tooltip higher up in the tree.
     */
    contextId?: string;
    /** @hidden */
    children: JSX.Element | ((props: TooltipRootChildrenProps) => JSX.Element);
};
/** Props that are passed to the Root component children callback. */
type TooltipRootChildrenProps = {
    /** Whether the tooltip is open. */
    open: boolean;
    /** Change the open state of the tooltip. */
    setOpen: Setter<boolean>;
    /** The initial placement of the tooltip. */
    placement: Placement;
    /** The strategy to use when positioning the tooltip. */
    strategy: Strategy;
    /** Floating options of the tooltip. */
    floatingOptions: FloatingOptions | null;
    /** The current floating state of the tooltip. */
    floatingState: FloatingState;
    /** The delay in milliseconds before the tooltip opens after the mouse pointer is moved over the trigger. */
    openDelay: number;
    /** The delay in milliseconds before the tooltip closes after the mouse pointer leaves. */
    closeDelay: number;
    /** The duration in milliseconds that the tooltip should ignore the `openDelay` when the mouse pointer leaves and re-enters. */
    skipDelayDuration: number;
    /** Whether the tooltip content should be hoverable. */
    hoverableContent: boolean;
    /** The group this tooltip is in. */
    group: true | string | null;
    /** Whether the tooltip should open when the trigger is focused. */
    openOnFocus: boolean;
    /** Whether the tooltip should open when the mouse pointer is moved over the trigger. */
    openOnHover: boolean;
    /** Whether the tooltip should close when the user presses the `Escape` key. */
    closeOnEscapeKeyDown: boolean;
    /** Whether the tooltip should close when the user presses on the trigger. */
    closeOnPointerDown: boolean;
    /** Whether the tooltip should close when the user scrolls. */
    closeOnScroll: boolean;
    /** Whether the tooltip content is present in the DOM. */
    contentPresent: boolean;
    /** The tooltip content element. */
    contentRef: HTMLElement | null;
    /** The `id` of the tooltip element. */
    tooltipId: string;
};
/** Context wrapper for the tooltip. Is required for every tooltip you create. */
declare const TooltipRoot: Component<TooltipRootProps>;

type TooltipTriggerCorvuProps = {
    /**
     * The `id` of the tooltip context to use.
     */
    contextId?: string;
};
type TooltipTriggerSharedElementProps<T extends ValidComponent = 'button'> = DynamicButtonSharedElementProps<T>;
type TooltipTriggerElementProps = TooltipTriggerSharedElementProps & {
    'aria-describedby': string | undefined;
    'aria-expanded': 'true' | 'false';
    'data-closed': '' | undefined;
    'data-open': '' | undefined;
    'data-placement': Placement | undefined;
    'data-corvu-tooltip-trigger': '';
} & DynamicButtonElementProps;
type TooltipTriggerProps<T extends ValidComponent = 'button'> = TooltipTriggerCorvuProps & Partial<TooltipTriggerSharedElementProps<T>>;
/** Button that opens the tooltip when focused or hovered.
 *
 * @data `data-corvu-tooltip-trigger` - Present on every tooltip trigger element.
 * @data `data-open` - Present when the tooltip is open.
 * @data `data-closed` - Present when the tooltip is closed.
 * @data `data-placement` - Current placement of the tooltip. Only present when the tooltip is open.
 */
declare const TooltipTrigger: <T extends ValidComponent = "button">(props: DynamicProps<T, TooltipTriggerProps<T>>) => solid_js.JSX.Element;

declare const Tooltip: solid_js.Component<TooltipRootProps> & {
    Anchor: <T extends solid_js.ValidComponent = "div">(props: DynamicProps<T, TooltipAnchorProps<T>>) => solid_js.JSX.Element;
    Trigger: <T extends solid_js.ValidComponent = "button">(props: DynamicProps<T, TooltipTriggerProps<T>>) => solid_js.JSX.Element;
    Portal: (props: TooltipPortalProps & solid_js.ComponentProps<typeof solid_js_web.Portal>) => solid_js.JSX.Element;
    Content: <T extends solid_js.ValidComponent = "div">(props: DynamicProps<T, TooltipContentProps<T>>) => solid_js.JSX.Element;
    Arrow: <T extends solid_js.ValidComponent = "div">(props: DynamicProps<T, TooltipArrowProps<T>>) => solid_js.JSX.Element;
    useContext: (contextId?: string) => TooltipContextValue;
};

export { TooltipAnchor as Anchor, type TooltipAnchorCorvuProps as AnchorCorvuProps, type TooltipAnchorElementProps as AnchorElementProps, type TooltipAnchorProps as AnchorProps, type TooltipAnchorSharedElementProps as AnchorSharedElementProps, TooltipArrow as Arrow, type TooltipArrowCorvuProps as ArrowCorvuProps, type TooltipArrowElementProps as ArrowElementProps, type TooltipArrowProps as ArrowProps, type TooltipArrowSharedElementProps as ArrowSharedElementProps, TooltipContent as Content, type TooltipContentCorvuProps as ContentCorvuProps, type TooltipContentElementProps as ContentElementProps, type TooltipContentProps as ContentProps, type TooltipContentSharedElementProps as ContentSharedElementProps, type TooltipContextValue as ContextValue, TooltipPortal as Portal, type TooltipPortalProps as PortalProps, TooltipRoot as Root, type TooltipRootChildrenProps as RootChildrenProps, type TooltipRootProps as RootProps, TooltipTrigger as Trigger, type TooltipTriggerCorvuProps as TriggerCorvuProps, type TooltipTriggerElementProps as TriggerElementProps, type TooltipTriggerProps as TriggerProps, type TooltipTriggerSharedElementProps as TriggerSharedElementProps, Tooltip as default, useTooltipContext as useContext };
