import { createComponent, mergeProps, Portal, memo } from 'solid-js/web';
import { createContext, useContext, splitProps, createMemo, Show, mergeProps as mergeProps$1, createUniqueId, createSignal, untrack } from 'solid-js';
import { Dynamic, DynamicButton } from '@corvu/utils/dynamic';
import { mergeRefs, some } from '@corvu/utils/reactivity';
import { useKeyedContext, createKeyedContext } from '@corvu/utils/create/keyedContext';
import FloatingArrow from '@corvu/utils/components/FloatingArrow';
import { combineStyle, callEventHandler } from '@corvu/utils/dom';
import { dataIf, isFunction } from '@corvu/utils';
import Dismissible from 'solid-dismissible';
import { getFloatingStyle } from '@corvu/utils/floating';
import createControllableSignal from '@corvu/utils/create/controllableSignal';
import createFloating from '@corvu/utils/create/floating';
import createOnce from '@corvu/utils/create/once';
import createPresence from 'solid-presence';
import createTooltip from '@corvu/utils/create/tooltip';

// src/Anchor.tsx
var TooltipContext = createContext();
var createTooltipContext = (contextId) => {
  if (contextId === undefined) return TooltipContext;
  const context = createKeyedContext(
    `tooltip-${contextId}`
  );
  return context;
};
var useTooltipContext = (contextId) => {
  if (contextId === undefined) {
    const context2 = useContext(TooltipContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Tooltip context not found. Make sure to wrap Tooltip components in <Tooltip.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(`tooltip-${contextId}`);
  if (!context) {
    throw new Error(
      `[corvu]: Tooltip context with id "${contextId}" not found. Make sure to wrap Tooltip components in <Tooltip.Root contextId="${contextId}">`
    );
  }
  return context;
};
var InternalTooltipContext = createContext();
var createInternalTooltipContext = (contextId) => {
  if (contextId === undefined) return InternalTooltipContext;
  const context = createKeyedContext(
    `tooltip-internal-${contextId}`
  );
  return context;
};
var useInternalTooltipContext = (contextId) => {
  if (contextId === undefined) {
    const context2 = useContext(InternalTooltipContext);
    if (!context2) {
      throw new Error(
        "[corvu]: Tooltip context not found. Make sure to wrap Tooltip components in <Tooltip.Root>"
      );
    }
    return context2;
  }
  const context = useKeyedContext(
    `tooltip-internal-${contextId}`
  );
  if (!context) {
    throw new Error(
      `[corvu]: Tooltip context with id "${contextId}" not found. Make sure to wrap Tooltip components in <Tooltip.Root contextId="${contextId}">`
    );
  }
  return context;
};

// src/Anchor.tsx
var TooltipAnchor = (props) => {
  const [localProps, otherProps] = splitProps(props, ["contextId", "ref"]);
  const context = createMemo(() => useInternalTooltipContext(localProps.contextId));
  return createComponent(Dynamic, mergeProps({
    as: "div",
    ref(r$) {
      var _ref$ = mergeRefs(context().setAnchorRef, localProps.ref);
      typeof _ref$ === "function" && _ref$(r$);
    },
    "data-corvu-tooltip-anchor": ""
  }, otherProps));
};
var Anchor_default = TooltipAnchor;
var TooltipArrow = (props) => {
  const [localProps, otherProps] = splitProps(props, ["contextId", "ref"]);
  const context = createMemo(() => useInternalTooltipContext(localProps.contextId));
  return createComponent(FloatingArrow, mergeProps({
    get floatingState() {
      return context().floatingState();
    },
    ref(r$) {
      var _ref$ = mergeRefs(context().setArrowRef, localProps.ref);
      typeof _ref$ === "function" && _ref$(r$);
    },
    "data-corvu-tooltip-arrow": ""
  }, otherProps));
};
var Arrow_default = TooltipArrow;
var TooltipContent = (props) => {
  const [localProps, otherProps] = splitProps(props, ["forceMount", "contextId", "ref", "style"]);
  const context = createMemo(() => useInternalTooltipContext(localProps.contextId));
  const show = () => some(context().open, () => localProps.forceMount, context().contentPresent);
  const enableDismissible = createMemo(() => context().open() || context().contentPresent());
  return createComponent(Dismissible, {
    get element() {
      return context().contentRef;
    },
    get enabled() {
      return enableDismissible();
    },
    get dismissibleId() {
      return context().tooltipId();
    },
    onDismiss: () => context().setOpen(false),
    get dismissOnEscapeKeyDown() {
      return context().closeOnEscapeKeyDown;
    },
    dismissOnOutsideFocus: false,
    dismissOnOutsidePointer: false,
    noOutsidePointerEvents: false,
    get onEscapeKeyDown() {
      return context().onEscapeKeyDown;
    },
    children: (props2) => createComponent(Show, {
      get when() {
        return show();
      },
      get children() {
        return createComponent(Dynamic, mergeProps({
          as: "div",
          ref(r$) {
            var _ref$ = mergeRefs(context().setContentRef, localProps.ref);
            typeof _ref$ === "function" && _ref$(r$);
          },
          get style() {
            return combineStyle({
              ...getFloatingStyle({
                strategy: () => context().strategy(),
                floatingState: () => context().floatingState()
              })(),
              "pointer-events": props2.isLastLayer ? "auto" : undefined
            }, localProps.style);
          },
          get id() {
            return context().tooltipId();
          },
          role: "tooltip",
          get ["data-closed"]() {
            return dataIf(!context().open());
          },
          get ["data-open"]() {
            return dataIf(context().open());
          },
          get ["data-placement"]() {
            return context().floatingState().placement;
          },
          "data-corvu-tooltip-content": ""
        }, otherProps));
      }
    })
  });
};
var Content_default = TooltipContent;
var TooltipPortal = (props) => {
  const [localProps, otherProps] = splitProps(props, ["forceMount", "contextId"]);
  const context = createMemo(() => useInternalTooltipContext(localProps.contextId));
  const show = () => some(context().open, () => localProps.forceMount, context().contentPresent);
  return createComponent(Show, {
    get when() {
      return show();
    },
    get children() {
      return createComponent(Portal, otherProps);
    }
  });
};
var Portal_default = TooltipPortal;
var TooltipRoot = (props) => {
  const defaultedProps = mergeProps$1({
    initialOpen: false,
    placement: "bottom",
    strategy: "absolute",
    floatingOptions: {
      flip: true,
      shift: true
    },
    openDelay: 500,
    closeDelay: 0,
    skipDelayDuration: 0,
    hoverableContent: true,
    group: null,
    openOnFocus: true,
    openOnHover: true,
    closeOnEscapeKeyDown: true,
    closeOnPointerDown: true,
    closeOnScroll: true,
    tooltipId: createUniqueId()
  }, props);
  const [open, setOpen] = createControllableSignal({
    value: () => defaultedProps.open,
    initialValue: defaultedProps.initialOpen,
    onChange: defaultedProps.onOpenChange
  });
  const [anchorRef, setAnchorRef] = createSignal(null);
  const [triggerRef, setTriggerRef] = createSignal(null);
  const [contentRef, setContentRef] = createSignal(null);
  const [arrowRef, setArrowRef] = createSignal(null);
  const {
    present: contentPresent
  } = createPresence({
    show: open,
    element: contentRef,
    onStateChange: (state) => {
      if (state === "present") {
        defaultedProps.onContentPresentChange?.(true);
      } else if (state === "hidden") {
        defaultedProps.onContentPresentChange?.(false);
      }
    }
  });
  const floatingState = createFloating({
    enabled: contentPresent,
    floating: contentRef,
    reference: () => anchorRef() ?? triggerRef() ?? null,
    arrow: arrowRef,
    placement: () => defaultedProps.placement,
    strategy: () => defaultedProps.strategy,
    options: () => defaultedProps.floatingOptions
  });
  createTooltip({
    id: () => defaultedProps.tooltipId,
    group: () => defaultedProps.group,
    open,
    close: () => setOpen(false),
    trigger: triggerRef,
    content: contentRef,
    openOnFocus: () => defaultedProps.openOnFocus,
    openOnHover: () => defaultedProps.openOnHover,
    closeOnPointerDown: () => defaultedProps.closeOnPointerDown,
    closeOnScroll: () => defaultedProps.closeOnScroll,
    hoverableContent: () => defaultedProps.hoverableContent,
    openDelay: () => defaultedProps.openDelay,
    closeDelay: () => defaultedProps.closeDelay,
    skipDelayDuration: () => defaultedProps.skipDelayDuration,
    onHover: (event) => {
      if (callEventHandler(defaultedProps.onHover, event)) return;
      setOpen(true);
    },
    onLeave: (event) => {
      if (callEventHandler(defaultedProps.onLeave, event)) return;
      setOpen(false);
    },
    onFocus: (event) => {
      if (callEventHandler(defaultedProps.onFocus, event)) return;
      setOpen(true);
    },
    onBlur: (event) => {
      if (callEventHandler(defaultedProps.onBlur, event)) return;
      setOpen(false);
    },
    onPointerDown: (event) => {
      if (callEventHandler(defaultedProps.onPointerDown, event)) return;
      setOpen(false);
    },
    onScroll: (event) => {
      if (callEventHandler(defaultedProps.onScroll, event)) return;
      setOpen(false);
    }
  });
  const childrenProps = {
    get open() {
      return open();
    },
    setOpen,
    get placement() {
      return defaultedProps.placement;
    },
    get strategy() {
      return defaultedProps.strategy;
    },
    get floatingOptions() {
      return defaultedProps.floatingOptions;
    },
    get floatingState() {
      return floatingState();
    },
    get openDelay() {
      return defaultedProps.openDelay;
    },
    get closeDelay() {
      return defaultedProps.closeDelay;
    },
    get skipDelayDuration() {
      return defaultedProps.skipDelayDuration;
    },
    get hoverableContent() {
      return defaultedProps.hoverableContent;
    },
    get group() {
      return defaultedProps.group;
    },
    get openOnFocus() {
      return defaultedProps.openOnFocus;
    },
    get openOnHover() {
      return defaultedProps.openOnHover;
    },
    get closeOnEscapeKeyDown() {
      return defaultedProps.closeOnEscapeKeyDown;
    },
    get closeOnPointerDown() {
      return defaultedProps.closeOnPointerDown;
    },
    get closeOnScroll() {
      return defaultedProps.closeOnScroll;
    },
    get contentPresent() {
      return contentPresent();
    },
    get contentRef() {
      return contentRef();
    },
    get tooltipId() {
      return defaultedProps.tooltipId;
    }
  };
  const memoizedChildren = createOnce(() => defaultedProps.children);
  const resolveChildren = () => {
    const children = memoizedChildren()();
    if (isFunction(children)) {
      return children(childrenProps);
    }
    return children;
  };
  const memoizedTooltipRoot = createMemo(() => {
    const TooltipContext2 = createTooltipContext(defaultedProps.contextId);
    const InternalTooltipContext2 = createInternalTooltipContext(defaultedProps.contextId);
    return untrack(() => createComponent(TooltipContext2.Provider, {
      value: {
        open,
        setOpen,
        placement: () => defaultedProps.placement,
        strategy: () => defaultedProps.strategy,
        floatingOptions: () => defaultedProps.floatingOptions,
        floatingState,
        openDelay: () => defaultedProps.openDelay,
        closeDelay: () => defaultedProps.closeDelay,
        skipDelayDuration: () => defaultedProps.skipDelayDuration,
        hoverableContent: () => defaultedProps.hoverableContent,
        group: () => defaultedProps.group,
        openOnFocus: () => defaultedProps.openOnFocus,
        openOnHover: () => defaultedProps.openOnHover,
        closeOnEscapeKeyDown: () => defaultedProps.closeOnEscapeKeyDown,
        closeOnPointerDown: () => defaultedProps.closeOnPointerDown,
        closeOnScroll: () => defaultedProps.closeOnScroll,
        contentPresent,
        contentRef,
        tooltipId: () => defaultedProps.tooltipId
      },
      get children() {
        return createComponent(InternalTooltipContext2.Provider, {
          get value() {
            return {
              open,
              setOpen,
              placement: () => defaultedProps.placement,
              strategy: () => defaultedProps.strategy,
              floatingOptions: () => defaultedProps.floatingOptions,
              floatingState,
              openDelay: () => defaultedProps.openDelay,
              closeDelay: () => defaultedProps.closeDelay,
              skipDelayDuration: () => defaultedProps.skipDelayDuration,
              hoverableContent: () => defaultedProps.hoverableContent,
              group: () => defaultedProps.group,
              openOnFocus: () => defaultedProps.openOnFocus,
              openOnHover: () => defaultedProps.openOnHover,
              closeOnEscapeKeyDown: () => defaultedProps.closeOnEscapeKeyDown,
              closeOnPointerDown: () => defaultedProps.closeOnPointerDown,
              closeOnScroll: () => defaultedProps.closeOnScroll,
              contentPresent,
              contentRef,
              tooltipId: () => defaultedProps.tooltipId,
              onFocus: defaultedProps.onFocus,
              onBlur: defaultedProps.onBlur,
              onPointerDown: defaultedProps.onPointerDown,
              onEscapeKeyDown: defaultedProps.onEscapeKeyDown,
              setAnchorRef,
              setTriggerRef,
              setContentRef,
              setArrowRef
            };
          },
          get children() {
            return untrack(() => resolveChildren());
          }
        });
      }
    }));
  });
  return memoizedTooltipRoot;
};
var Root_default = TooltipRoot;
var TooltipTrigger = (props) => {
  const [localProps, otherProps] = splitProps(props, ["contextId", "ref"]);
  const context = createMemo(() => useInternalTooltipContext(localProps.contextId));
  return createComponent(DynamicButton, mergeProps({
    ref(r$) {
      var _ref$ = mergeRefs(context().setTriggerRef, localProps.ref);
      typeof _ref$ === "function" && _ref$(r$);
    },
    get ["aria-describedby"]() {
      return memo(() => !!context().open())() ? context().tooltipId() : undefined;
    },
    get ["aria-expanded"]() {
      return context().open() ? "true" : "false";
    },
    get ["data-closed"]() {
      return dataIf(!context().open());
    },
    get ["data-open"]() {
      return dataIf(context().open());
    },
    get ["data-placement"]() {
      return memo(() => !!context().open())() ? context().floatingState().placement : undefined;
    },
    "data-corvu-tooltip-trigger": ""
  }, otherProps));
};
var Trigger_default = TooltipTrigger;

// src/index.ts
var Tooltip = Object.assign(Root_default, {
  Anchor: Anchor_default,
  Trigger: Trigger_default,
  Portal: Portal_default,
  Content: Content_default,
  Arrow: Arrow_default,
  useContext: useTooltipContext
});
var index_default = Tooltip;

export { Anchor_default as Anchor, Arrow_default as Arrow, Content_default as Content, Portal_default as Portal, Root_default as Root, Trigger_default as Trigger, index_default as default, useTooltipContext as useContext };
