'use strict';

var core = require('@felte/core');
var solidJs = require('solid-js');

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol, Iterator */


function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

function createAccessor(store) {
    const subscribed = {};
    const signals = {};
    function felteAccessor(selectorOrPath) {
        if (!selectorOrPath)
            return store();
        if (!subscribed[selectorOrPath.toString()]) {
            const valueSignal = solidJs.createSignal(core.getValue(solidJs.untrack(store), selectorOrPath));
            signals[selectorOrPath.toString()] = valueSignal;
            subscribed[selectorOrPath.toString()] = selectorOrPath;
        }
        const [value] = signals[selectorOrPath.toString()];
        return value();
    }
    solidJs.createEffect(() => {
        const storeValue = store();
        const keys = Object.keys(subscribed);
        for (const key of keys) {
            const subscriber = subscribed[key];
            const newValue = core.getValue(storeValue, subscriber);
            const [value, setValue] = signals[subscriber.toString()];
            if (core.isEqual(newValue, solidJs.untrack(value)))
                continue;
            setValue(newValue);
        }
    });
    return felteAccessor;
}

function createSubscriber(signal) {
    return function subscribe(fn) {
        return solidJs.createRoot((dispose) => {
            solidJs.createEffect(() => fn(signal()));
            return dispose;
        });
    };
}
const storeFactory = (initialValue) => {
    const [signal, setSignal] = solidJs.createSignal(initialValue);
    function signalSetter(value) {
        setSignal(() => value);
    }
    function signalUpdater(updater) {
        setSignal(updater);
    }
    const accessor = createAccessor(signal);
    const subscribe = createSubscriber(signal);
    accessor.subscribe = subscribe;
    accessor.set = signalSetter;
    accessor.update = signalUpdater;
    return accessor;
};

function createForm(config) {
    const _a = core.createForm(config !== null && config !== void 0 ? config : {}, {
        storeFactory,
    }), { form: formAction, cleanup, startStores, data, errors, warnings, touched } = _a, rest = __rest(_a, ["form", "cleanup", "startStores", "data", "errors", "warnings", "touched"]);
    let _destroy = null;
    const destroy = () => _destroy === null || _destroy === void 0 ? void 0 : _destroy();
    const [node, setNode] = solidJs.createSignal(null);
    solidJs.createEffect(() => {
        const _node = node();
        if (_node) {
            destroy();
            _destroy = formAction(_node).destroy;
        }
    });
    const form = (_node) => {
        setNode(_node);
        return {
            destroy,
        };
    };
    solidJs.onCleanup(destroy);
    solidJs.onCleanup(cleanup);
    return Object.assign(Object.assign({}, rest), { data: data, errors: errors, warnings: warnings, touched: touched, form });
}

function createField(nameOrConfig, config) {
    const { field: coreField, onChange, onInput, onBlur } = core.createField(nameOrConfig, config);
    function field(node) {
        const { destroy } = coreField(node);
        /* istanbul ignore next */
        solidJs.onCleanup(() => destroy === null || destroy === void 0 ? void 0 : destroy());
        return { destroy };
    }
    return {
        field,
        onChange,
        onInput,
        onBlur,
    };
}

Object.defineProperty(exports, "FelteSubmitError", {
    enumerable: true,
    get: function () { return core.FelteSubmitError; }
});
exports.createField = createField;
exports.createForm = createForm;
//# sourceMappingURL=index.cjs.map
