import { use, template } from 'solid-js/web';
import ApexCharts from 'apexcharts/dist/apexcharts.esm.js';
import { mergeProps, onMount, createEffect, on, onCleanup } from 'solid-js';
import { unwrap } from 'solid-js/store';
import { defu } from 'defu';

// src/SolidApexCharts.tsx
var _tmpl$ = /* @__PURE__ */ template(`<div>`);
var SolidApexCharts = (props) => {
  let rootEl;
  let chart;
  const merged = mergeProps({
    height: "auto",
    width: "100%",
    series: [],
    type: "line"
  }, props);
  const init = () => {
    const newOptions = {
      chart: {
        type: merged.type,
        height: merged.height,
        width: merged.width,
        events: {}
      },
      series: unwrap(merged.series)
    };
    for (const key in props) {
      if (key.startsWith("on")) {
        const eventKey = key.charAt(2).toLowerCase() + key.slice(3);
        newOptions.chart.events[eventKey] = props[key];
      }
    }
    const config = defu(unwrap(merged.options), newOptions);
    chart = new ApexCharts(rootEl, config);
    props.ref?.(chart);
    chart.render();
  };
  onMount(() => {
    init();
  });
  createEffect(on(() => merged.series, (series) => {
    chart.updateSeries(series);
  }, {
    defer: true
  }));
  createEffect(on(() => merged.options, (options) => {
    chart.updateOptions(options);
  }, {
    defer: true
  }));
  createEffect(on(() => [merged.type, merged.height, merged.width], () => {
    chart.destroy();
    init();
  }, {
    defer: true
  }));
  onCleanup(() => {
    chart.destroy();
  });
  return (() => {
    var _el$ = _tmpl$();
    var _ref$ = rootEl;
    typeof _ref$ === "function" ? use(_ref$, _el$) : rootEl = _el$;
    return _el$;
  })();
};
var SolidApexCharts_default = SolidApexCharts;

export { SolidApexCharts_default as SolidApexCharts };
